@@ -28,6 +28,13 @@ This code is dedicated to my daughter, mone.
 #include <boost/regex.hpp>
 #include <qfile.h>
 #include <qtextedit.h>
+//#include <qdatastream.h>
+#include <qlineedit.h>
+//#include <qtimer.h>
+#include <qprocess.h>
+//#include <qprogressbar.h>
+//#include <qprogressdialog.h>
+//#include <qapplication.h>
 /*for segate_show_status()*/
 #include <selinux/selinux.h>
 #include <selinux/get_default_type.h>
@@ -47,23 +54,49 @@ time_t the_time;
 long log_from_time = 0;
 int log_from_time_int = 0;
 
+QProcess *process;
+QProcess *process1;
+//QProgressBar* progress_bar;
+//QProgressDialog* pd;
+//QTimer progress_bar_timer;
+
+
 #define MAX_LOG 8460
 
 /*prototype declaration*/
 int segatex_set_time_real();
 int segatex_set_time_all_real();
+int segatex_cat_files();
 const char* segatex_show_installed_policy();
 void segatex_fixfiles_relabel();
 void segatex_show_status();
 void segatex_set_time();
 void segatex_set_time_all();
+void segatex_yum_install();
 void segatex_yum_update();
 void segatex_module_list();
 void segatex_load_from_my_policy();
+void segatex_load_from_file_contexts();
+void segatex_load_from_file_contexts_local();
+void segatex_ls_laZ();
+void segatex_write_file_contexts_local();
 void segatex_remove_my_policy();
 void segatex_write_to_my_policy();
 void segatex_show_yum_update();
+void segatex_show_help();
 void segatex_show_interface();
+/*
+void segatex_qtimer();
+void segatex_advance_progress_bar();
+void segatex_stop_progress_bar();
+void operation();
+*/
+void updateOutputTextEdit();
+void updateOutputTextEdit1();
+void processExited();
+void processExited1();
+void readFromStdout();
+void readFromStdout1();
 
 const char* segatex::segatex_show_installed_policy()
 {
@@ -97,7 +130,8 @@ const char* segatex::segatex_show_installed_policy()
 void segatex::segatex_show_status()
 {
         using namespace std;
-        const char* filename_sestatus="./segatex_sestatus";
+        //const char* filename_sestatus="./segatex_sestatus";
+        const char* filename_sestatus="/root/segatex/segatex_sestatus";
         ofstream os;
         os.open(filename_sestatus);
         if(os){
@@ -178,7 +212,8 @@ void segatex::segatex_remove_my_policy()
 {
         using namespace std;
         #define PWD_BUF 8600
-        const char* remove_file="./remove_echo";
+        //const char* remove_file="./remove_echo";
+        const char* remove_file="/root/segatex/remove_echo";
         ofstream os;
         os.open(remove_file);
         if(os){
@@ -187,7 +222,7 @@ void segatex::segatex_remove_my_policy()
                 char buffer[SIZE_TMP];
                 int chars_read;
                 memset(buffer, '\0', sizeof(buffer));
-                read_fp = popen("semodule -r my_policy_by_segatex", "r");
+                read_fp = popen("/usr/sbin/semodule -r my_policy_by_segatex", "r");
                 if(read_fp != NULL){
                         chars_read = fread(buffer, sizeof(char), SIZE_TMP, read_fp);
                         while(chars_read>0){
@@ -208,11 +243,31 @@ void segatex::segatex_remove_my_policy()
         }
 }
 
+void segatex::segatex_show_help()
+{
+        using namespace std;
+        const char* help_file="/root/segatex/HELP";
+	ifstream is;
+	is.open(help_file);
+	if(is){
+		QFile file( help_file ); 
+		if ( file.open( IO_ReadOnly ) ) {
+			QTextStream stream( &file );
+			textEdit1->setText( stream.read() );
+		}
+		is.close();
+	}else{
+		;//do nothing
+	}
+
+}
+
 void segatex::segatex_module_list()
 {
         using namespace std;
         #define PWD_BUF 8600
-        const char* module_list_file="./module_list";
+        //const char* module_list_file="./module_list";
+        const char* module_list_file="/root/segatex/module_list";
         ofstream os;
         os.open(module_list_file);
         if(os){
@@ -221,7 +276,7 @@ void segatex::segatex_module_list()
                 char buffer[SIZE_TMP];
                 int chars_read;
                 memset(buffer, '\0', sizeof(buffer));
-                read_fp = popen("semodule -l", "r");
+                read_fp = popen("/usr/sbin/semodule -l", "r");
                 if(read_fp != NULL){
                         chars_read = fread(buffer, sizeof(char), SIZE_TMP, read_fp);
                         while(chars_read>0){
@@ -241,10 +296,149 @@ void segatex::segatex_module_list()
         }
 }
 
+//just opens a file 
+void segatex::segatex_load_from_file_contexts()
+{
+	using namespace std;
+	const char* system_policy=segatex::segatex_show_installed_policy();
+	string system_policy_str=system_policy;
+        string file_contexts_file_pre="/etc/selinux/";
+        string file_contexts_file_last="/contexts/files/file_contexts";
+        string file_contexts_string=file_contexts_file_pre + system_policy_str + file_contexts_file_last;
+        const char* file_contexts=file_contexts_string.c_str();
+	ifstream is;
+	is.open(file_contexts);
+	if(is){
+		QFile file( file_contexts ); 
+		if ( file.open( IO_ReadOnly ) ) {
+			QTextStream stream( &file );
+			textEdit3->setText( stream.read() );
+		}
+		is.close();
+	}else{
+		;//do nothing
+	}
+}
+
+//opens a file for editing
+void segatex::segatex_load_from_file_contexts_local()
+{
+	using namespace std;
+	const char* system_policy=segatex::segatex_show_installed_policy();
+	string system_policy_str=system_policy;
+        string file_contexts_file_pre_local="/etc/selinux/";
+        string file_contexts_file_last_local="/contexts/files/file_contexts.local";
+        string file_contexts_string_local=file_contexts_file_pre_local + system_policy_str + file_contexts_file_last_local;
+        const char* file_contexts_local=file_contexts_string_local.c_str();
+	ifstream is;
+	is.open(file_contexts_local);
+	if(is){
+		QFile file( file_contexts_local ); 
+		if ( file.open( IO_ReadOnly ) ) {
+			QTextStream stream( &file );
+                        textEdit4->setText( stream.read() );
+		}
+		is.close();
+	}else{
+		;//do nothing
+	}
+}
+
+//write to a file
+void segatex::segatex_write_file_contexts_local()
+{
+	using namespace std;
+	const char* system_policy=segatex::segatex_show_installed_policy();
+	string system_policy_str=system_policy;
+	string file_contexts_file_pre_local="/etc/selinux/";
+        string file_contexts_file_last_local="/contexts/files/file_contexts.local";
+        string touch_com="touch ";
+        string file_contexts_string_local=file_contexts_file_pre_local + system_policy_str + file_contexts_file_last_local;
+        string touch_file_contexts_string_local=touch_com + file_contexts_file_pre_local + system_policy_str + file_contexts_file_last_local;
+        const char* touch_file_contexts_local=touch_file_contexts_string_local.c_str();
+        const char* file_contexts_local=file_contexts_string_local.c_str();
+	const char* touch_fl="/root/segatex/touch_file_contexts_local";
+        ofstream os_pre;
+	os_pre.open(touch_fl);
+	if(os_pre){
+                FILE *read_fp;
+                #define SIZE_TMP 8600
+                char buffer[SIZE_TMP];
+                int chars_read;
+                memset(buffer, '\0', sizeof(buffer));
+                read_fp = popen(touch_file_contexts_local, "r");
+                if(read_fp != NULL){
+                        chars_read = fread(buffer, sizeof(char), SIZE_TMP, read_fp);
+                        while(chars_read>0){
+                                buffer[chars_read -1] = '\0';
+                                //os << "Reading:-" << endl;
+                                os_pre << buffer;
+                                chars_read= fread(buffer, sizeof(char), SIZE_TMP, read_fp);
+                        }
+                        pclose(read_fp);
+                }
+	}
+	ifstream is;
+	is.open(file_contexts_local);
+	if(is){
+		QFile file( file_contexts_local ); 
+		if ( file.open( IO_ReadWrite ) ) {
+			QTextStream stream( &file );
+                        stream << textEdit4->text();
+                        textEdit4->setModified( FALSE );
+		}
+		is.close();
+	}else{
+		textEdit4->setText("a file does not exists");
+		;//do nothing
+	}
+}
+
+//get lineEdit1 and pass it to process
+void segatex::segatex_ls_laZ()
+{
+	using namespace std;
+	QString str;
+	str=lineEdit1->text();
+	const char* str_char="";
+	str_char = str;
+	string str_process_pre= "ls -laZ ";
+	string str_process= str_process_pre + str_char;
+
+        const char* ls_laZ_file="/root/segatex/ls_laZ";
+        ofstream os;
+        os.open(ls_laZ_file);
+        if(os){
+                FILE *read_fp;
+                #define SIZE_TMP 8600
+                char buffer[SIZE_TMP];
+                int chars_read;
+                memset(buffer, '\0', sizeof(buffer));
+                read_fp = popen(str_process.c_str(), "r");
+                if(read_fp != NULL){
+                        chars_read = fread(buffer, sizeof(char), SIZE_TMP, read_fp);
+                        while(chars_read>0){
+                                buffer[chars_read -1] = '\0';
+                                //os << "Reading:-" << endl;
+                                os << buffer;
+                                chars_read= fread(buffer, sizeof(char), SIZE_TMP, read_fp);
+                        }
+                        pclose(read_fp);
+                }
+                os.close();
+                QFile file( ls_laZ_file ); // Read the text from a file
+                if ( file.open( IO_ReadOnly ) ) {
+                        QTextStream stream( &file );
+                        textEdit3->setText( stream.read() );
+                }
+        }
+}
+
 void segatex::segatex_load_from_my_policy()
 {
 	using namespace std;
-        const char* filename_policy="./my_policy_by_segatex.te";
+        //const char* filename_policy="./my_policy_by_segatex.te";
+        const char* filename_policy="/root/segatex/my_policy_by_segatex.te";
 	ifstream is;
 	is.open(filename_policy);
 	if(is){
@@ -262,8 +456,10 @@ void segatex::segatex_load_from_my_policy()
 void segatex::segatex_write_to_my_policy()
 {
 	using namespace std;
-	const char* my_policy_file="./my_policy_by_segatex.te";
-	const char* load_my_policy_file="./load_my_policy";
+	//const char* my_policy_file="./my_policy_by_segatex.te";
+	const char* my_policy_file="/root/segatex/my_policy_by_segatex.te";
+	//const char* load_my_policy_file="./load_my_policy";
+	const char* load_my_policy_file="/root/segatex/load_my_policy";
 	ifstream os_pre;
 	os_pre.open(my_policy_file);
 	if(os_pre){
@@ -287,7 +483,8 @@ void segatex::segatex_write_to_my_policy()
 				char buffer[SIZE_TMP];
 				int chars_read;
 				memset(buffer, '\0', sizeof(buffer));
-				read_fp_install = popen("make -f /usr/share/selinux/devel/Makefile", "r");
+				//read_fp_install = popen("make -f /usr/share/selinux/devel/Makefile", "r");
+				read_fp_install = popen("make -C /root/segatex -f /usr/share/selinux/devel/Makefile", "r");
 				if(read_fp_install != NULL){
 					chars_read = fread(buffer, sizeof(char), SIZE_TMP, read_fp_install);
 					while(chars_read>0){
@@ -298,7 +495,8 @@ void segatex::segatex_write_to_my_policy()
 					}
 					pclose(read_fp_install);
 				}
-				read_fp = popen("semodule -i my_policy_by_segatex.pp", "r");
+				//read_fp = popen("semodule -i my_policy_by_segatex.pp", "r");
+				read_fp = popen("/usr/sbin/semodule -i /root/segatex/my_policy_by_segatex.pp", "r");
 				if(read_fp != NULL){
 					chars_read = fread(buffer, sizeof(char), SIZE_TMP, read_fp);
 					while(chars_read>0){
@@ -320,77 +518,178 @@ void segatex::segatex_write_to_my_policy()
 		}
 	}
 }
+
 void segatex::segatex_show_yum_update()
 {
         status_label_1->setText("yum install/updating SELinx related rpms and audit rpm. Please wait for a while...");
 }
 
-void segatex::segatex_yum_update()
+/*
+void segatex::segatex_qtimer()
 {
-	using namespace std;
-	#define PWD_BUF 8600
-	const char* yum_update_file="./yum_update";
-	ofstream os;
-	os.open(yum_update_file);
-	if(os){
-		FILE *read_fp_install;
-		FILE *read_fp;
-		FILE *read_fp_reload_base_module;
-		#define SIZE_TMP 8600
-		char buffer[SIZE_TMP];
-		int chars_read;
-		memset(buffer, '\0', sizeof(buffer));
-		read_fp_install = popen("yum -y install audit selinux-policy* libsepol libselinux libsemanage policycoreutils checkpolicy", "r");
-		if(read_fp_install != NULL){
-			chars_read = fread(buffer, sizeof(char), SIZE_TMP, read_fp_install);
-			while(chars_read>0){
-				buffer[chars_read -1] = '\0';
-				//os << "Reading:-" << endl;
-				os << buffer;
-				chars_read= fread(buffer, sizeof(char), SIZE_TMP, read_fp_install);
-			}
-			pclose(read_fp_install);
-		}
-		read_fp = popen("yum -y update audit selinux-policy* libsepol libselinux libsemanage policycoreutils checkpolicy", "r");
-		if(read_fp != NULL){
-			chars_read = fread(buffer, sizeof(char), SIZE_TMP, read_fp);
-			while(chars_read>0){
-				buffer[chars_read -1] = '\0';
-				//os << "Reading:-" << endl;
-				os << buffer;
-				chars_read= fread(buffer, sizeof(char), SIZE_TMP, read_fp);
-			}
-			pclose(read_fp);
-		}
-		const char* installed_policy=segatex::segatex_show_installed_policy();
-		string installed_policy_str=installed_policy;
-		string message_for_reload_base_module_pre= "Reloaded base module from ";
-		string popen_for_reload_base_module_pre= "semodule -b /usr/share/selinux/";
-		string popen_for_reload_base_module_path= "/usr/share/selinux/";
-		string popen_for_reload_base_module_tail= "/base.pp";
-		string message_for_reload_base_module= message_for_reload_base_module_pre + popen_for_reload_base_module_path + installed_policy_str + popen_for_reload_base_module_tail;
-		string popen_for_reload_base_module= popen_for_reload_base_module_pre + installed_policy + popen_for_reload_base_module_tail;
-		const char* please=popen_for_reload_base_module.c_str();
-		read_fp_reload_base_module=popen(please,"r");
-		if(read_fp_reload_base_module != NULL){
-			os << endl << endl << message_for_reload_base_module << endl;
-		}
-		pclose(read_fp_reload_base_module);
-		os.close();
-		QFile file( yum_update_file ); // Read the text from a file
-		if ( file.open( IO_ReadOnly ) ) {
-			QTextStream stream( &file );
-			textEdit1->setText( stream.read() );
-		}
-	}
+	//for progressBar
+	progress_bar_timer.start(200, false);
+	connect( &progress_bar_timer, SIGNAL( timeout() ),
+		     this, SLOT( segatex_advance_progress_bar()) );
 }
 
+void segatex::segatex_advance_progress_bar()
+{
+	progressBar1->setProgress(progressBar1->progress() + 10);
+	qApp->processEvents();
+}
+
+void segatex::segatex_stop_progress_bar()
+{
+	progress_bar_timer.stop();
+	progressBar1->setProgress(0);
+}
+
+void segatex::operation()
+{
+	int totalSteps = 1000;
+	QProgressDialog progress( "Install and updating SELinux related RPMs...", 0 ,  totalSteps,
+				  this, "progress", TRUE );
+	for ( int i = 0; i < totalSteps; i++ ) {
+	    progress.setProgress( i );
+	    qApp->processEvents();
+	    if ( progress.wasCanceled() )
+		break;
+	}
+	progress.setProgress( totalSteps );
+}
+*/
+
+void segatex::readFromStdout()
+{
+	// Read and process the data.
+	// Bear in mind that the data might be output in chunks.
+	QByteArray data = process->readStdout();
+	QString text = textEdit1->text() + QString(data);
+	textEdit1->setText(text);
+}
+
+void segatex::readFromStdout1()
+{
+	// Read and process the data.
+	// Bear in mind that the data might be output in chunks.
+	QByteArray data = process1->readStdout();
+	QString text = textEdit1->text() + QString(data);
+	textEdit1->setText(text);
+}
+
+void segatex::updateOutputTextEdit()
+{
+	// Read and process the data.
+	// Bear in mind that the data might be output in chunks.
+	QByteArray data = process->readStderr();
+	QString text = textEdit1->text() + QString(data);
+	textEdit1->setText(text);
+}
+
+void segatex::updateOutputTextEdit1()
+{
+	// Read and process the data.
+	// Bear in mind that the data might be output in chunks.
+	QByteArray data = process1->readStderr();
+	QString text = textEdit1->text() + QString(data);
+	textEdit1->setText(text);
+}
+
+void segatex::processExited()
+{
+	if(process->normalExit()){
+		textEdit1->append(tr("yum install process ended !"));
+		textEdit1->append(tr("\n"));
+	} else {
+		textEdit1->append(tr("yum install process had an error !"));
+		textEdit1->append(tr("\n"));
+	}
+	delete process;
+	process = 0;
+	textEdit1->append(tr("yum update process will start please wait"));
+	textEdit1->append(tr("\n"));
+	segatex_yum_update();
+}
+
+void segatex::processExited1()
+{
+	if(process1->normalExit()){
+		textEdit1->append(tr("yum update process ended !"));
+		textEdit1->append(tr("\n"));
+	} else {
+		textEdit1->append(tr("yum update process had an error !"));
+		textEdit1->append(tr("\n"));
+	}
+	delete process1;
+	process1 = 0;
+	yum_install_update->setEnabled(true);
+}
+
+void segatex::segatex_yum_install()
+{
+	yum_install_update->setEnabled(false);
+	process = new QProcess(this);
+	process->addArgument("yum");
+	process->addArgument("-y");
+	process->addArgument("install");
+	process->addArgument("audit");
+	process->addArgument("selinux-policy");
+	process->addArgument("selinux-policy-devel");
+	process->addArgument("selinux-policy-mls");
+	process->addArgument("selinux-policy-strict");
+	process->addArgument("selinux-policy-targeted");
+	process->addArgument("libsepol");
+	process->addArgument("libselinux");
+	process->addArgument("libselinux-devel");
+	process->addArgument("libselinux-python");
+	process->addArgument("libsemanage");
+	process->addArgument("policycoreutils");
+	process->addArgument("checkpolicy");
+	//connect(process, SIGNAL(readyReadStdout()),this,SLOT(segatex_advance_progress_bar()));
+	connect(process, SIGNAL(readyReadStdout()),this,SLOT(readFromStdout()));
+	connect(process, SIGNAL(readyReadStderr()),this,SLOT(updateOutputTextEdit()));
+	connect(process, SIGNAL(processExited()),this,SLOT(processExited()));
+	//for progressbar to stop 
+	//connect(process, SIGNAL(processExited()),this,SLOT(segatex_stop_progress_bar()));
+	process->start();
+}
+
+void segatex::segatex_yum_update()
+{
+	yum_install_update->setEnabled(false);
+	process1 = new QProcess(this);
+	process1->addArgument("yum");
+	process1->addArgument("-y");
+	process1->addArgument("update");
+	process1->addArgument("audit");
+	process1->addArgument("selinux-policy");
+	process1->addArgument("selinux-policy-devel");
+	process1->addArgument("selinux-policy-mls");
+	process1->addArgument("selinux-policy-strict");
+	process1->addArgument("selinux-policy-targeted");
+	process1->addArgument("libsepol");
+	process1->addArgument("libselinux");
+	process1->addArgument("libselinux-devel");
+	process1->addArgument("libselinux-python");
+	process1->addArgument("libsemanage");
+	process1->addArgument("policycoreutils");
+	process1->addArgument("checkpolicy");
+	//connect(process1, SIGNAL(readyReadStdout()),this,SLOT(segatex_advance_progress_bar()));
+	connect(process1, SIGNAL(readyReadStdout()),this,SLOT(readFromStdout1()));
+	connect(process1, SIGNAL(readyReadStderr()),this,SLOT(updateOutputTextEdit1()));
+	connect(process1, SIGNAL(processExited()),this,SLOT(processExited1()));
+	//for progressbar to stop 
+	//connect(process1, SIGNAL(processExited()),this,SLOT(segatex_stop_progress_bar()));
+	process1->start();
+}
 
 void segatex::segatex_audit_2_allow()
 {
 	using namespace std;
 	#define PWD_BUF 8600
-	const char* audit_2_allow_file="./local.te";
+	//const char* audit_2_allow_file="./local.te";
+	const char* audit_2_allow_file="/root/segatex/local.te";
 	ofstream os;
 	os.open(audit_2_allow_file);
 	if(os){
@@ -399,7 +698,7 @@ void segatex::segatex_audit_2_allow()
 		char buffer[SIZE_TMP];
 		int chars_read;
 		memset(buffer, '\0', sizeof(buffer));
-		read_fp = popen("audit2allow -m local -a", "r");
+		read_fp = popen("/usr/bin/audit2allow -m local -a", "r");
 		if(read_fp != NULL){
 			chars_read = fread(buffer, sizeof(char), SIZE_TMP, read_fp);
 			while(chars_read>0){
@@ -419,11 +718,42 @@ void segatex::segatex_audit_2_allow()
 	}
 }
 
+int segatex::segatex_cat_files()
+{
+	int cat_files_state = 1;
+	using namespace std;
+	#define PWD_BUF 8600
+	const char* avc_file="/root/segatex/avc_file";
+	ofstream os;
+	os.open(avc_file);
+	if(os){
+		FILE *read_fp;
+		#define SIZE_TMP 8600
+		char buffer[SIZE_TMP];
+		int chars_read;
+		memset(buffer, '\0', sizeof(buffer));
+		read_fp = popen("cat /var/log/messages /var/log/audit/audit.log > /root/segatex/audit_messages.log", "r");
+		if(read_fp != NULL){
+			chars_read = fread(buffer, sizeof(char), SIZE_TMP, read_fp);
+			while(chars_read>0){
+				buffer[chars_read -1] = '\0';
+				os << buffer;
+				chars_read= fread(buffer, sizeof(char), SIZE_TMP, read_fp);
+			}
+			pclose(read_fp);
+			cat_files_state = 0;
+		}
+		os.close();
+	}
+	return cat_files_state;
+}
+
 void segatex::segatex_audit_2_why()
 {
 	using namespace std;
 	#define PWD_BUF 8600
-	const char* audit_2_why_file="./local_te_why";
+	//const char* audit_2_why_file="./local_te_why";
+	const char* audit_2_why_file="/root/segatex/local_te_why";
 	ofstream os;
 	os.open(audit_2_why_file);
 	if(os){
@@ -432,7 +762,7 @@ void segatex::segatex_audit_2_why()
 		char buffer[SIZE_TMP];
 		int chars_read;
 		memset(buffer, '\0', sizeof(buffer));
-		read_fp = popen("audit2why < /var/log/audit/audit.log", "r");
+		read_fp = popen("/usr/sbin/audit2why < /var/log/audit/audit.log", "r");
 		if(read_fp != NULL){
 			chars_read = fread(buffer, sizeof(char), SIZE_TMP, read_fp);
 			while(chars_read>0){
@@ -480,9 +810,9 @@ void segatex::segatex_show_interface()
 
 	using namespace std;
 	using namespace boost;
-	const char* filename="/var/log/audit/audit.log";
-	const char* filename_my_log="./audit_log_mine_must_be_here";
-        const char* filename_policy="./my_policy_by_segatex.te";
+	const char* filename="";
+	const char* filename_my_log="/root/segatex/audit_log_mine_must_be_here";
+        const char* filename_policy="/root/segatex/my_policy_by_segatex.te";
 	if(log_state == 0){
 		log_from_time_int = segatex::segatex_set_time_all_real(); 
 	}
@@ -493,7 +823,7 @@ void segatex::segatex_show_interface()
         char buffer_c[MAX_LOG];
 
 	//read from engine and store them to v_engine
-        const char* filename_engine="./selpaw_interface.csv";
+        const char* filename_engine="/root/segatex/selpaw_interface.csv";
 
         ifstream is_engine;
         ofstream os_engine_test;
@@ -525,6 +855,12 @@ void segatex::segatex_show_interface()
 			//os_engine_test << buffer_engine << endl;
 		}while(!is_engine.eof());
 		//os_engine_test.close();
+		int avc_state = segatex::segatex_cat_files();
+		if(avc_state == 0 ){
+			filename="/root/segatex/audit_messages.log";
+		}else{
+			os_policy << "cannot read from log !" << endl;	
+		}
 		ifstream is;
 		is.open(filename);
 		//When log is missing.
@@ -652,9 +988,9 @@ void segatex::segatex_show_interface()
 					const char* den = "denied";
 					const char* log_time = "audit\\([0-9]{10}";
 					const char* scontext = "scontext([^[:blank:]]*)([:blank:])?";
-					const char* scontext_rev = "t_[a-zA-Z0-9_]([^:]*)?";
+					const char* scontext_rev = "t_[a-zA-Z0-9_.]([^:]*)?";
 					const char* tcontext = "tcontext([^[:blank:]]*)([:blank:])?";
-					const char* tcontext_rev = "t_[a-zA-Z0-9_]([^:]*)?";
+					const char* tcontext_rev = "t_[a-zA-Z0-9_.]([^:]*)?";
 					//changed in segatex2.11
 					//const char* tclass = "tclass(.*)";
 					const char* tclass = "tclass=[a-zA-Z0-9_]*";
@@ -863,8 +1199,8 @@ void segatex::segatex_show_interface()
 					p2_v_end = unique(v2.begin(),v2.end());
 					const char* m_pre = "(.*):([^[:space:]]*)([:space:])?";
 					const char* m_interface_pre = "[a-z]([^[:space:]]*):([^[:space:]]*)([:space:])?";
-					const char* m_domain_pre = "[A-Za-z0-9_]*_t";
-					const char* m_tclass_pre = "[A-Za-z0-9_]*_t:";
+					const char* m_domain_pre = "[A-Za-z0-9_.]*_t";
+					const char* m_tclass_pre = "[A-Za-z0-9_.]*_t:";
 					//changed here at segatex2.11
 					const char* m_objclass_pre = ":[a-zA-Z0-9_]*([^[:space:]]*)([:space:])?";
 					const char* m_acc = "\\{(.*)\\}";
@@ -1454,9 +1790,9 @@ void segatex::segatex_show_interface()
 
 
 
-					const char* m_domain_interface_h = "[a-zA-Z][a-zA-Z0-9_]*_t:";
+					const char* m_domain_interface_h = "[a-zA-Z][a-zA-Z0-9_.]*_t:";
 					const char* m_domain_interface_h_204 = "allow(.*)_t";
-					const char* m_domain_interface_h_204_2 = "[[:space:]][a-zA-Z][a-zA-Z0-9_]*_t";
+					const char* m_domain_interface_h_204_2 = "[[:space:]][a-zA-Z][a-zA-Z0-9_.]*_t";
 					const char* m_objclass_interface_h = ":.*?\\{";
 					const char* m_objclass_interface_h_dash = ":[a-zA-Z0-9_]*";
 					const char* m_objclass_interface_h2 = ":(.*)([^[:space:]*])[[:space:]]";
@@ -1582,7 +1918,7 @@ void segatex::segatex_show_interface()
 
 								//segatex2.05ɮʤ󡢤ޤʡ
 								regex tmp_re("[[:space:]]([^[:space:]]*)[a-zA-Z0-9_]*");
-								regex tmp_re2("[[:space:]][a-zA-Z0-9_]*_t");
+								regex tmp_re2("[[:space:]][a-zA-Z0-9_.]*_t");
 								string::const_iterator it=accv_interface_h.begin(), end=accv_interface_h.end();
 								while(it != end-1){
 									smatch result;
@@ -1662,7 +1998,7 @@ void segatex::segatex_show_interface()
 					p_tmp_objclass_end=unique(v_tmp_objclass.begin(),v_tmp_objclass.end());
 
 					os_policy << "####################################################" << endl;
-					os_policy << "##### This file was generated by segatex2.15" << endl;
+					os_policy << "##### This file was generated by segatex3.02" << endl;
 					os_policy << "####################################################" << endl;
 					os_policy << "module my_policy_by_segatex 1.0.0;" << endl << endl;
 					os_policy << "require {" << endl;
